@@ -48,20 +48,28 @@
 // CairoOutputDev
 //------------------------------------------------------------------------
 
-CairoOutputDev::CairoOutputDev(void) {
+CairoOutputDev::CairoOutputDev() {
   xref = NULL;
 
   FT_Init_FreeType(&ft_lib);
   fontEngine = NULL;
+  glyphs = NULL;
+  surface = NULL;
 }
 
 CairoOutputDev::~CairoOutputDev() {
   if (fontEngine) {
     delete fontEngine;
   }
-  cairo_destroy (cairo);
   FT_Done_FreeType(ft_lib);
+  cairo_surface_destroy (surface);
+}
 
+void CairoOutputDev::setSurface(cairo_surface_t *surface)
+{
+  cairo_surface_destroy (this->surface);
+  cairo_surface_reference (surface);
+  this->surface = surface;
 }
 
 void CairoOutputDev::startDoc(XRef *xrefA) {
@@ -73,20 +81,11 @@ void CairoOutputDev::startDoc(XRef *xrefA) {
 }
 
 void CairoOutputDev::startPage(int pageNum, GfxState *state) {
-  cairo_destroy (cairo);
-  createCairo (state);
-  
-  cairo_init_clip (cairo);
-  cairo_set_rgb_color (cairo, 0, 0, 0);
-  cairo_set_operator (cairo, CAIRO_OPERATOR_OVER);
-  cairo_set_line_cap (cairo, CAIRO_LINE_CAP_BUTT);
-  cairo_set_line_join (cairo, CAIRO_LINE_JOIN_MITER);
-  cairo_set_dash (cairo, NULL, 0, 0.0);
-  cairo_set_miter_limit (cairo, 10);
-  cairo_set_tolerance (cairo, 1);
+  cairo = cairo_create (surface);
 }
 
 void CairoOutputDev::endPage() {
+  cairo_destroy (cairo);
 }
 
 void CairoOutputDev::drawLink(Link *link, Catalog *catalog) {
@@ -100,8 +99,13 @@ void CairoOutputDev::saveState(GfxState *state) {
 void CairoOutputDev::restoreState(GfxState *state) {
   LOG(printf ("restore\n"));
   cairo_restore (cairo);
-  /* TODO: Is this really needed for cairo? Maybe not */
-  needFontUpdate = gTrue;
+
+  /* These aren't restored by cairo_restore() since we keep them in
+   * the output device. */
+  updateFillColor(state);
+  updateStrokeColor(state);
+  updateFillOpacity(state);
+  updateStrokeOpacity(state);
 }
 
 void CairoOutputDev::updateAll(GfxState *state) {
@@ -113,6 +117,8 @@ void CairoOutputDev::updateAll(GfxState *state) {
   updateMiterLimit(state);
   updateFillColor(state);
   updateStrokeColor(state);
+  updateFillOpacity(state);
+  updateStrokeOpacity(state);
   needFontUpdate = gTrue;
 }
 
@@ -146,7 +152,7 @@ void CairoOutputDev::updateLineDash(GfxState *state) {
 }
 
 void CairoOutputDev::updateFlatness(GfxState *state) {
-  cairo_set_tolerance (cairo, state->getFlatness());
+  // cairo_set_tolerance (cairo, state->getFlatness());
 }
 
 void CairoOutputDev::updateLineJoin(GfxState *state) {
@@ -196,30 +202,48 @@ void CairoOutputDev::updateStrokeColor(GfxState *state) {
   LOG(printf ("stroke color: %f %f %f\n", stroke_color.r, stroke_color.g, stroke_color.b));
 }
 
+void CairoOutputDev::updateFillOpacity(GfxState *state) {
+  fill_opacity = state->getFillOpacity();
+  LOG(printf ("fill opacity: %f\n", fill_opacity));
+}
+
+void CairoOutputDev::updateStrokeOpacity(GfxState *state) {
+  stroke_opacity = state->getStrokeOpacity();
+  LOG(printf ("stroke opacity: %f\n", stroke_opacity));
+}
+
 void CairoOutputDev::updateFont(GfxState *state) {
-  cairo_font_t *font;
+  cairo_font_face_t *font_face;
   double m11, m12, m21, m22;
   double w;
+  cairo_matrix_t matrix;
 
   LOG(printf ("updateFont() font=%s\n", state->getFont()->getName()->getCString()));
   
-  /* Needs to be rethough, since fonts are now handled by cairo */
   needFontUpdate = gFalse;
 
+  currentFont = fontEngine->getFont (state->getFont(), xref);
+
   state->getFontTransMat(&m11, &m12, &m21, &m22);
   m11 *= state->getHorizScaling();
   m12 *= state->getHorizScaling();
 
-  /* w = currentFont->getSubstitutionCorrection(state->getFont()); */
-  m12 *= -1;
-  m22 *= -1;
+  w = currentFont->getSubstitutionCorrection(state->getFont());
+  m12 *= w;
+  m22 *= w;
 
   LOG(printf ("font matrix: %f %f %f %f\n", m11, m12, m21, m22));
   
-  currentFont = fontEngine->getFont (state->getFont(), xref,
-				     m11, m21, m12, m22);
-  font = currentFont->getFont();
-  cairo_set_font (cairo, font);
+  font_face = currentFont->getFontFace();
+  cairo_set_font_face (cairo, font_face);
+
+  matrix.xx = m11;
+  matrix.xy = -m21;
+  matrix.yx = m12;
+  matrix.yy = -m22;
+  matrix.x0 = 0;
+  matrix.y0 = 0;
+  cairo_set_font_matrix (cairo, &matrix);
 }
 
 void CairoOutputDev::doPath(GfxState *state, GfxPath *path,
@@ -240,14 +264,14 @@ void CairoOutputDev::doPath(GfxState *state, GfxPath *path,
       j = 1;
       while (j < subpath->getNumPoints()) {
 	if (subpath->getCurve(j)) {
+	  state->transform(subpath->getX(j), subpath->getY(j), &x1, &y1);
+	  state->transform(subpath->getX(j+1), subpath->getY(j+1), &x2, &y2);
+	  state->transform(subpath->getX(j+2), subpath->getY(j+2), &x3, &y3);
 	  if (snapToGrid) {
 	    x1 = round (x1); y1 = round (y1);
 	    x2 = round (x2); y2 = round (y2);
 	    x3 = round (x3); y3 = round (y3);
 	  }
-	  state->transform(subpath->getX(j), subpath->getY(j), &x1, &y1);
-	  state->transform(subpath->getX(j+1), subpath->getY(j+1), &x2, &y2);
-	  state->transform(subpath->getX(j+2), subpath->getY(j+2), &x3, &y3);
 	  cairo_curve_to (cairo, 
 			  x1, y1,
 			  x2, y2,
@@ -274,8 +298,9 @@ void CairoOutputDev::doPath(GfxState *state, GfxPath *path,
 
 void CairoOutputDev::stroke(GfxState *state) {
   doPath (state, state->getPath(), gFalse);
-  cairo_set_rgb_color (cairo,
-		       stroke_color.r, stroke_color.g, stroke_color.b);
+  cairo_set_source_rgba (cairo,
+			 stroke_color.r, stroke_color.g, stroke_color.b,
+			 stroke_opacity);
   LOG(printf ("stroke\n"));
   cairo_stroke (cairo);
 }
@@ -283,8 +308,9 @@ void CairoOutputDev::stroke(GfxState *state) {
 void CairoOutputDev::fill(GfxState *state) {
   doPath (state, state->getPath(), gFalse);
   cairo_set_fill_rule (cairo, CAIRO_FILL_RULE_WINDING);
-  cairo_set_rgb_color (cairo,
-		       fill_color.r, fill_color.g, fill_color.b);
+  cairo_set_source_rgba (cairo,
+			 fill_color.r, fill_color.g, fill_color.b,
+			 fill_opacity);
   LOG(printf ("fill\n"));
   cairo_fill (cairo);
 }
@@ -292,17 +318,16 @@ void CairoOutputDev::fill(GfxState *state) {
 void CairoOutputDev::eoFill(GfxState *state) {
   doPath (state, state->getPath(), gFalse);
   cairo_set_fill_rule (cairo, CAIRO_FILL_RULE_EVEN_ODD);
-  cairo_set_rgb_color (cairo,
+  cairo_set_source_rgb (cairo,
 		       fill_color.r, fill_color.g, fill_color.b);
   LOG(printf ("fill-eo\n"));
   cairo_fill (cairo);
 }
 
-void CairoOutputDev::clip(GfxState *state, GBool snapToGrid) {
-  doPath (state, state->getPath(), snapToGrid);
+void CairoOutputDev::clip(GfxState *state) {
+  doPath (state, state->getPath(), gFalse);
   cairo_set_fill_rule (cairo, CAIRO_FILL_RULE_WINDING);
   cairo_clip (cairo);
-  cairo_new_path (cairo); /* Consume path */
   LOG (printf ("clip\n"));
 }
 
@@ -310,108 +335,66 @@ void CairoOutputDev::eoClip(GfxState *state) {
   doPath (state, state->getPath(), gFalse);
   cairo_set_fill_rule (cairo, CAIRO_FILL_RULE_EVEN_ODD);
   cairo_clip (cairo);
-  cairo_new_path (cairo); /* Consume path */
   LOG (printf ("clip-eo\n"));
 }
 
-void CairoOutputDev::drawString(GfxState *state, GooString *s)
+void CairoOutputDev::beginString(GfxState *state, GooString *s)
 {
-  GfxFont *font;
-  int wMode;
-  int render;
-  // the number of bytes in the string and not the number of glyphs?
   int len = s->getLength();
-  // need at most len glyphs
-  cairo_glyph_t *glyphs;
-  
-  char *p = s->getCString();
-  int count = 0;
-  double curX, curY;
-  double riseX, riseY;
 
-  font = state->getFont();
-  wMode = font->getWMode();
- 
-  if (needFontUpdate) {
+  if (needFontUpdate)
     updateFont(state);
-  }
-  if (!currentFont) {
+
+  glyphs = (cairo_glyph_t *) gmalloc (len * sizeof (cairo_glyph_t));
+  glyphCount = 0;
+}
+
+void CairoOutputDev::drawChar(GfxState *state, double x, double y,
+			      double dx, double dy,
+			      double originX, double originY,
+			      CharCode code, Unicode *u, int uLen)
+{
+  double tx, ty;
+
+  glyphs[glyphCount].index = currentFont->getGlyph (code, u, uLen);
+  state->transform(x, y, &tx, &ty);
+  glyphs[glyphCount].x = tx;
+  glyphs[glyphCount].y = ty;
+  glyphCount++;
+}
+
+void CairoOutputDev::endString(GfxState *state)
+{
+  int render;
+
+  if (!currentFont)
     return;
-  }
    
   // check for invisible text -- this is used by Acrobat Capture
   render = state->getRender();
-  if (render == 3) {
+  if (render == 3)
     return;
-  }
 
   // ignore empty strings
-  if (len == 0)
+  if (glyphCount == 0) {
+    gfree(glyphs);
+    glyphs = NULL;
     return;
-  
-  glyphs = (cairo_glyph_t *) gmalloc (len * sizeof (cairo_glyph_t));
-
-  state->textTransformDelta(0, state->getRise(), &riseX, &riseY);
-  curX = state->getCurX();
-  curY = state->getCurY();
-  while (len > 0) {
-    double x, y;
-    double x1, y1;
-    double dx, dy, tdx, tdy;
-    double originX, originY, tOriginX, tOriginY;
-    int n, uLen;
-    CharCode code;
-    Unicode u[8];
-    n = font->getNextChar(p, len, &code,
-	                  u, (int)(sizeof(u) / sizeof(Unicode)), &uLen,
-			  &dx, &dy, &originX, &originY);
-    if (wMode) {
-      dx *= state->getFontSize();
-      dy = dy * state->getFontSize() + state->getCharSpace();
-      if (n == 1 && *p == ' ') {
-	dy += state->getWordSpace();
-      }
-    } else {
-      dx = dx * state->getFontSize() + state->getCharSpace();
-      if (n == 1 && *p == ' ') {
-	dx += state->getWordSpace();
-      }
-      dx *= state->getHorizScaling();
-      dy *= state->getFontSize();
-    }
-    originX *= state->getFontSize();
-    originY *= state->getFontSize();
-    state->textTransformDelta(dx, dy, &tdx, &tdy);
-    state->textTransformDelta(originX, originY, &tOriginX, &tOriginY);
-    x = curX + riseX;
-    y = curY + riseY;
-    x -= tOriginX;
-    y -= tOriginY;
-    state->transform(x, y, &x1, &y1);
-
-    glyphs[count].index = currentFont->getGlyph (code, u, uLen);
-    glyphs[count].x = x1;
-    glyphs[count].y = y1;
-    curX += tdx;
-    curY += tdy;
-    p += n;
-    len -= n;
-    count++;
   }
-  // fill
+  
   if (!(render & 1)) {
     LOG (printf ("fill string\n"));
-    cairo_set_rgb_color (cairo,
+    cairo_set_source_rgb (cairo,
 			 fill_color.r, fill_color.g, fill_color.b);
-    cairo_show_glyphs (cairo, glyphs, count);
+    cairo_show_glyphs (cairo, glyphs, glyphCount);
   }
   
   // stroke
   if ((render & 3) == 1 || (render & 3) == 2) {
     LOG (printf ("stroke string\n"));
-    cairo_set_rgb_color (cairo,
+    cairo_set_source_rgb (cairo,
 			 stroke_color.r, stroke_color.g, stroke_color.b);
-    cairo_glyph_path (cairo, glyphs, count);
+    cairo_glyph_path (cairo, glyphs, glyphCount);
     cairo_stroke (cairo);
   }
 
@@ -421,11 +404,12 @@ void CairoOutputDev::drawString(GfxState *state, GooString *s)
     // glyphs within one text object before we clip.  Right now this
     // just add this one string.
     LOG (printf ("clip string\n"));
-    cairo_glyph_path (cairo, glyphs, count);
+    cairo_glyph_path (cairo, glyphs, glyphCount);
     cairo_clip (cairo);
   }
   
   gfree (glyphs);
+  glyphs = NULL;
 }
 
 GBool CairoOutputDev::beginType3Char(GfxState *state, double x, double y,
@@ -451,17 +435,20 @@ void CairoOutputDev::endTextObject(GfxState *state) {
 void CairoOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 				    int width, int height, GBool invert,
 				    GBool inlineImg) {
-  char *buffer;
-  char *dest;
+  unsigned char *buffer;
+  unsigned char *dest;
   cairo_surface_t *image;
+  cairo_pattern_t *pattern;
   int x, y;
   ImageStream *imgStr;
   Guchar *pix;
   double *ctm;
-  cairo_matrix_t *mat;
+  cairo_matrix_t matrix;
   int invert_bit;
+  int row_stride;
 
-  buffer = (char *)malloc (width * height * 4);
+  row_stride = (width + 3) & ~3;
+  buffer = (unsigned char *) malloc (height * row_stride);
   if (buffer == NULL) {
     error(-1, "Unable to allocate memory for image.");
     return;
@@ -475,13 +462,9 @@ void CairoOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 
   for (y = 0; y < height; y++) {
     pix = imgStr->getLine();
-    dest = buffer + y * width * 4;
+    dest = buffer + y * row_stride;
     for (x = 0; x < width; x++) {
 
-      *dest++ = soutRound(255 * fill_color.b);
-      *dest++ = soutRound(255 * fill_color.g);
-      *dest++ = soutRound(255 * fill_color.r);
- 
       if (pix[x] ^ invert_bit)
 	*dest++ = 0;
       else
@@ -489,58 +472,56 @@ void CairoOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
     }
   }
 
-  cairo_save (cairo);
+  image = cairo_image_surface_create_for_data (buffer, CAIRO_FORMAT_A8,
+					       width, height, row_stride);
+  if (image == NULL)
+    return;
+  pattern = cairo_pattern_create_for_surface (image);
+  if (pattern == NULL)
+    return;
 
   ctm = state->getCTM();
-  mat = cairo_matrix_create ();
   LOG (printf ("drawImageMask %dx%d, matrix: %f, %f, %f, %f, %f, %f\n",
-	       width, height,
-	       ctm[0], ctm[1],
-	       ctm[2], ctm[3],
-	       ctm[4], ctm[5]));
-  cairo_matrix_set_affine (mat,
-			   ctm[0]/width, ctm[1]/width,
-			   -ctm[2]/height, -ctm[3]/height,
-			   ctm[2] + ctm[4], ctm[3] + ctm[5]);
-  cairo_concat_matrix (cairo, mat);
-  cairo_matrix_destroy (mat);
+	       width, height, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]));
+  matrix.xx = ctm[0] / width;
+  matrix.xy = -ctm[2] / height;
+  matrix.yx = ctm[1] / width;
+  matrix.yy = -ctm[3] / height;
+  matrix.x0 = ctm[2] + ctm[4];
+  matrix.y0 = ctm[3] + ctm[5];
+  cairo_matrix_invert (&matrix);
+  cairo_pattern_set_matrix (pattern, &matrix);
 
-  image = cairo_surface_create_for_image (
-              buffer, CAIRO_FORMAT_ARGB32, width, height, width * 4);
-  cairo_surface_set_filter (image, CAIRO_FILTER_BEST);
-  cairo_show_surface (cairo, image, width, height);
-
-  cairo_restore (cairo);
+  cairo_pattern_set_filter (pattern, CAIRO_FILTER_BEST);
+  /* FIXME: Doesn't the image mask support any colorspace? */
+  cairo_set_source_rgb (cairo, fill_color.r, fill_color.g, fill_color.b);
+  cairo_mask (cairo, pattern);
 
+  cairo_pattern_destroy (pattern);
   cairo_surface_destroy (image);
   free (buffer);
   delete imgStr;
-
-  
 }
 
 void CairoOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 				int width, int height,
 				GfxImageColorMap *colorMap,
-				int *maskColors, GBool inlineImg) {
-  char *buffer;
-  char *dest;
+				int *maskColors, GBool inlineImg)
+{
+  unsigned char *buffer;
+  unsigned int *dest;
   cairo_surface_t *image;
+  cairo_pattern_t *pattern;
   int x, y;
   ImageStream *imgStr;
   Guchar *pix;
   GfxRGB rgb;
   int alpha, i;
   double *ctm;
-  cairo_matrix_t *mat;
+  cairo_matrix_t matrix;
   int is_identity_transform;
   
-  buffer = (char *)malloc (width * height * 4);
-
-  if (buffer == NULL) {
-    error(-1, "Unable to allocate memory for image.");
-    return;
-  }
+  buffer = (unsigned char *)gmalloc (width * height * 4);
 
   /* TODO: Do we want to cache these? */
   imgStr = new ImageStream(str, width,
@@ -553,60 +534,65 @@ void CairoOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   is_identity_transform = colorMap->getColorSpace()->getMode() == csDeviceRGB ||
 		  colorMap->getColorSpace()->getMode() == csICCBased && 
 		  ((GfxICCBasedColorSpace*)colorMap->getColorSpace())->getAlt()->getMode() == csDeviceRGB;
-  
-  for (y = 0; y < height; y++) {
-    dest = buffer + y * 4 * width;
-    pix = imgStr->getLine();
-    for (x = 0; x < width; x++, pix += colorMap->getNumPixelComps()) {
-      if (maskColors) {
-	alpha = 0;
+
+  if (maskColors) {
+    for (y = 0; y < height; y++) {
+      dest = (unsigned int *) (buffer + y * 4 * width);
+      pix = imgStr->getLine();
+      colorMap->getRGBLine (pix, dest, width);
+
+      for (x = 0; x < width; x++) {
 	for (i = 0; i < colorMap->getNumPixelComps(); ++i) {
-	  if (pix[i] < maskColors[2*i] ||
-	      pix[i] > maskColors[2*i+1]) {
-	    alpha = 255;
+	  
+	  if (pix[i] < maskColors[2*i] * 255||
+	      pix[i] > maskColors[2*i+1] * 255) {
+	    *dest = *dest | 0xff000000;
 	    break;
 	  }
 	}
-      } else {
-	alpha = 255;
+	pix += colorMap->getNumPixelComps();
+	dest++;
       }
-      if (is_identity_transform) {
-	*dest++ = pix[2];
-	*dest++ = pix[1];
-	*dest++ = pix[0];
-      } else {      
-	colorMap->getRGB(pix, &rgb);
-	*dest++ = soutRound(255 * rgb.b);
-	*dest++ = soutRound(255 * rgb.g);
-	*dest++ = soutRound(255 * rgb.r);
-      }
-      *dest++ = alpha;
     }
+
+    image = cairo_image_surface_create_for_data (buffer, CAIRO_FORMAT_ARGB32,
+						 width, height, width * 4);
+  }
+  else {
+    for (y = 0; y < height; y++) {
+      dest = (unsigned int *) (buffer + y * 4 * width);
+      pix = imgStr->getLine();
+      colorMap->getRGBLine (pix, dest, width);
+    }
+
+    image = cairo_image_surface_create_for_data (buffer, CAIRO_FORMAT_RGB24,
+						 width, height, width * 4);
   }
 
-  cairo_save (cairo);
+  if (image == NULL)
+    return;
+  pattern = cairo_pattern_create_for_surface (image);
+  if (pattern == NULL)
+    return;
 
   ctm = state->getCTM();
-  mat = cairo_matrix_create ();
-  LOG (printf ("draw image %dx%d, matrix: %f, %f, %f, %f, %f, %f\n",
-	       width, height,
-	       ctm[0], ctm[1],
-	       ctm[2], ctm[3],
-	       ctm[4], ctm[5]));
-  cairo_matrix_set_affine (mat,
-			   ctm[0]/width, ctm[1]/width,
-			   -ctm[2]/height, -ctm[3]/height,
-			   ctm[2] + ctm[4], ctm[3] + ctm[5]);
-  cairo_concat_matrix (cairo, mat);
-  cairo_matrix_destroy (mat);
-  
-  image = cairo_surface_create_for_image (
-              buffer, CAIRO_FORMAT_ARGB32, width, height, width * 4);
-  cairo_surface_set_filter (image, CAIRO_FILTER_BEST);
-  cairo_show_surface (cairo, image, width, height);
+  LOG (printf ("drawImageMask %dx%d, matrix: %f, %f, %f, %f, %f, %f\n",
+	       width, height, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]));
+  matrix.xx = ctm[0] / width;
+  matrix.xy = -ctm[2] / height;
+  matrix.yx = ctm[1] / width;
+  matrix.yy = -ctm[3] / height;
+  matrix.x0 = ctm[2] + ctm[4];
+  matrix.y0 = ctm[3] + ctm[5];
 
-  cairo_restore (cairo);
-  
+  cairo_matrix_invert (&matrix);
+  cairo_pattern_set_matrix (pattern, &matrix);
+
+  cairo_pattern_set_filter (pattern, CAIRO_FILTER_BILINEAR);
+  cairo_set_source (cairo, pattern);
+  cairo_paint (cairo);
+
+  cairo_pattern_destroy (pattern);
   cairo_surface_destroy (image);
   free (buffer);
   delete imgStr;
