@@ -36,7 +36,7 @@ class CairoOutputDev: public OutputDev {
 public:
 
   // Constructor.
-  CairoOutputDev(void);
+  CairoOutputDev();
 
   // Destructor.
   virtual ~CairoOutputDev();
@@ -48,7 +48,7 @@ public:
   virtual GBool upsideDown() { return gTrue; }
 
   // Does this device use drawChar() or drawString()?
-  virtual GBool useDrawChar() { return gFalse; }
+  virtual GBool useDrawChar() { return gTrue; }
 
   // Does this device use beginType3Char/endType3Char?  Otherwise,
   // text in Type 3 fonts will be drawn with drawChar/drawString.
@@ -81,6 +81,8 @@ public:
   virtual void updateLineWidth(GfxState *state);
   virtual void updateFillColor(GfxState *state);
   virtual void updateStrokeColor(GfxState *state);
+  virtual void updateFillOpacity(GfxState *state);
+  virtual void updateStrokeOpacity(GfxState *state);
 
   //----- update text state
   virtual void updateFont(GfxState *state);
@@ -91,11 +93,17 @@ public:
   virtual void eoFill(GfxState *state);
 
   //----- path clipping
-  virtual void clip(GfxState *state, GBool snapToGrid);
+  virtual void clip(GfxState *state);
   virtual void eoClip(GfxState *state);
 
   //----- text drawing
-  virtual void drawString(GfxState *state, GooString *s);
+  void beginString(GfxState *state, GooString *s);
+  void endString(GfxState *state);
+  void drawChar(GfxState *state, double x, double y,
+		double dx, double dy,
+		double originX, double originY,
+		CharCode code, Unicode *u, int uLen);
+
   virtual GBool beginType3Char(GfxState *state, double x, double y,
 			       double dx, double dy,
 			       CharCode code, Unicode *u, int uLen);
@@ -122,22 +130,26 @@ public:
  
   GBool isReverseVideo() { return gFalse; }
   
+  void setSurface (cairo_surface_t *surface);
+
 protected:
-  virtual void createCairo(GfxState *state) = 0;
   void doPath(GfxState *state, GfxPath *path, GBool snapToGrid);
   
   GfxRGB fill_color;
   GfxRGB stroke_color;
+  double fill_opacity;
+  double stroke_opacity;
   CairoFont *currentFont;
   
   XRef *xref;			// xref table for current document
 
   FT_Library ft_lib;
   CairoFontEngine *fontEngine;
-  unsigned char *pixels;
-  int pixels_w, pixels_h;
   cairo_t *cairo;
-  GBool needFontUpdate;		// set when the font needs to be updated
+  GBool needFontUpdate;                // set when the font needs to be updated
+  cairo_surface_t *surface;
+  cairo_glyph_t *glyphs;
+  int glyphCount;
 };
 
 #endif
