@@ -33,6 +33,14 @@
 #include "JPXStream.h"
 #include "Stream-CCITT.h"
 
+#ifdef ENABLE_LIBJPEG
+#include "DCTStream.h"
+#endif
+
+#ifdef ENABLE_ZLIB
+#include "FlateStream.h"
+#endif
+
 #ifdef __DJGPP__
 static GBool setDJSYSFLAGS = gFalse;
 #endif
@@ -1786,6 +1794,8 @@ GBool CCITTFaxStream::isBinary(GBool last) {
   return str->isBinary(gTrue);
 }
 
+#ifndef ENABLE_LIBJPEG
+
 //------------------------------------------------------------------------
 // DCTStream
 //------------------------------------------------------------------------
@@ -3177,6 +3187,9 @@ GBool DCTStream::isBinary(GBool last) {
   return str->isBinary(gTrue);
 }
 
+#endif
+
+#ifndef ENABLE_ZLIB
 //------------------------------------------------------------------------
 // FlateStream
 //------------------------------------------------------------------------
@@ -3707,6 +3720,7 @@ int FlateStream::getCodeWord(int bits) {
   codeSize -= bits;
   return c;
 }
+#endif
 
 //------------------------------------------------------------------------
 // EOFStream
