@@ -532,6 +532,7 @@ private:
   void eatBits(int n) { inputBits -= n; }
 };
 
+#ifndef ENABLE_LIBJPEG
 //------------------------------------------------------------------------
 // DCTStream
 //------------------------------------------------------------------------
@@ -632,7 +633,9 @@ private:
   int readMarker();
   int read16();
 };
+#endif
 
+#ifndef ENABLE_ZLIB
 //------------------------------------------------------------------------
 // FlateStream
 //------------------------------------------------------------------------
@@ -707,6 +710,7 @@ private:
   int getHuffmanCodeWord(FlateHuffmanTab *tab);
   int getCodeWord(int bits);
 };
+#endif
 
 //------------------------------------------------------------------------
 // EOFStream
