@@ -1,5 +1,6 @@
 /* poppler-qt.h: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
+ * Copyright (C) 2005, Tobias Koening
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,9 @@
 
 namespace Poppler {
 
+class Document;
+class Page;
+
 /* A rectangle on a page, with coordinates in PDF points. */
 class Rectangle
 {
@@ -39,6 +43,76 @@ class Rectangle
     double m_y2;
 };
 
+class PageTransitionData;
+class PageTransition
+{
+friend class Page;
+public:
+  enum Type {
+    Replace,
+    Split,
+    Blinds,
+    Box,
+    Wipe,
+    Dissolve,
+    Glitter,
+    Fly,
+    Push,
+    Cover,
+    Uncover,
+    Fade
+  };
+
+  enum Alignment {
+    Horizontal,
+    Vertical
+  };
+
+  enum Direction {
+    Inward,
+    Outward
+  };
+
+
+  // Destructor
+  ~PageTransition();
+
+  // Get type of the transition.
+  Type getType() const { return type; }
+
+  // Get duration of the transition in seconds.
+  int getDuration() const { return duration; }
+
+  // Get dimension in which the transition effect
+  // occurs.
+  Alignment getAlignment() const { return alignment; }
+
+  // Get direction of motion of the transition effect.
+  Direction getDirection() const { return direction; }
+
+  // Get direction in which the transition effect moves.
+  int getAngle() const { return angle; }
+
+  // Get starting or ending scale.
+  double getScale() const { return scale; }
+
+  // Returns true if the area to be flown is rectangular and
+  // opaque.
+  bool isRectangular() const { return rectangular; }
+
+private:
+  // Construct a new PageTransition object from a page dictionary.
+  PageTransition( const PageTransitionData &data );
+
+  Type type;
+  int duration;
+  Alignment alignment;
+  Direction direction;
+  int angle;
+  double scale;
+  bool rectangular;
+};
+
 class PageData;
 class Page {
   friend class Document;
@@ -51,6 +125,12 @@ class Page {
     * If r is a null Rectangle all text of the page is given
     **/
     QString getText(const Rectangle &r) const;
+
+    /**
+    * Returns the transition of this page
+    **/
+    PageTransition *getTransition() const;
+    
   private:
     Page(const Document *doc, int index);
     PageData *data;
@@ -78,7 +158,7 @@ public:
   
   PageMode getPageMode() const;
   
-  bool unlock(QCString &password);
+  bool unlock(const QCString &password);
   
   bool isLocked() const;
   
