@@ -220,7 +220,7 @@ SplashPath *SplashFTFont::getGlyphPath(int c) {
   } else {
     gid = (FT_UInt)c;
   }
-  if (FT_Load_Glyph(ff->face, gid, FT_LOAD_DEFAULT)) {
+  if (FT_Load_Glyph(ff->face, gid, FT_LOAD_NO_BITMAP)) {
     return NULL;
   }
   if (FT_Get_Glyph(slot, &glyph)) {
